package com.gree.mall.manager.bean.material.stock;

import com.gree.mall.manager.commonmapper.MaterialMapper;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.plus.entity.WebsitPartsNewRefundManageRecord;
import com.gree.mall.manager.plus.entity.WebsitPartsSalesItem;
import com.gree.mall.manager.plus.service.WebsitPartsSalesItemService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionSynchronization;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class PartsStockLogic {

    private final RedisLockRegistry redisLockRegistry;
    private final MaterialMapper materialMapper;
    private final WebsitPartsSalesItemService websitPartsSalesItemService;

    /**
     * 恢复新件销售数量
     * @param
     */
    public void revertNewPartsSalesQty(String websitId, String partsWebsitId, String identity,
                                       List<WebsitPartsNewRefundManageRecord> partsNewRefundManageRecordList) throws Exception {
        // 锁定销售单明细
        String lockName = websitId + "|" + partsWebsitId + "|" + identity;
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.WRITE_SALES_ITEM + lockName);
        if(!obtain.tryLock(10, TimeUnit.SECONDS)){
            throw new RemoteServiceException("销售明细已冻结:" + lockName);
        }
        // 处理销售单明细
        try{
            // 查询销售的明细
            List<WebsitPartsSalesItem> salesOrderItems = materialMapper.querySalesItemByItemId(websitId, partsWebsitId,
                    identity, partsNewRefundManageRecordList.stream()
                            .map(WebsitPartsNewRefundManageRecord::getSalesItemId)
                            .collect(Collectors.toList()));

            Map<String, WebsitPartsSalesItem> salesOrderItemMap = salesOrderItems.stream()
                    .collect(Collectors.toMap(WebsitPartsSalesItem::getId, Function.identity()));
            for (WebsitPartsNewRefundManageRecord partsNewRefundManageRecord : partsNewRefundManageRecordList) {
                WebsitPartsSalesItem partsSalesOrderItem = salesOrderItemMap.get(partsNewRefundManageRecord.getSalesItemId());
                partsSalesOrderItem.setNewRefundQty(partsSalesOrderItem.getNewRefundQty().subtract(partsNewRefundManageRecord.getQty()));
                partsSalesOrderItem.setQty(partsSalesOrderItem.getQty().add(partsNewRefundManageRecord.getQty()));
            }

            // 更新销售明细数据
            websitPartsSalesItemService.saveOrUpdateBatch(salesOrderItemMap.values());

        } finally {
            this.txCallUnlockSalesItem(obtain);
        }
    }

    private void txCallUnlockSalesItem(Lock obtain) {
        //事务提交完成后才释放锁,此方法禁止执行mysql相关操作,否则将导致重大问题。
        if (TransactionSynchronizationManager.isActualTransactionActive()) {
            TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronization() {
                @Override
                public void afterCompletion(int status) {

                    if(TransactionSynchronization.STATUS_COMMITTED == status){
                        log.info("=========【销售单明细】======事务提交==============");
                    }else if(TransactionSynchronization.STATUS_ROLLED_BACK == status){
                        log.info("========【销售单明细】=========事务回滚============");
                    }

                    obtain.unlock();
                }
            });
        } else {
            log.info("【销售单明细】====没有事务===释放锁");
            obtain.unlock();
        }
    }
}
